@@ -1,4 +1,4 @@
 from __future__ import unicode_literals
 from netmiko.a10.a10_ssh import A10SSH
 
-__all__ = ['A10SSH']
+__all__ = ["A10SSH"]
