@@ -1,4 +1,5 @@
 from __future__ import unicode_literals
+import time
 from netmiko.base_connection import BaseConnection
 
 
@@ -7,6 +8,7 @@ class CheckPointGaiaSSH(BaseConnection):
     Implements methods for communicating with Check Point Gaia
     firewalls.
     """
+
     def session_preparation(self):
         """
         Prepare the session after the connection has been established.
@@ -15,12 +17,19 @@ class CheckPointGaiaSSH(BaseConnection):
         """
         self._test_channel_read()
         self.set_base_prompt()
-        self.disable_paging(command="set clienv rows 0\n")
+        self.disable_paging(command="set clienv rows 0")
+        # Clear the read buffer
+        time.sleep(0.3 * self.global_delay_factor)
+        self.clear_buffer()
 
-    def config_mode(self, config_command=''):
+    def config_mode(self, config_command=""):
         """No config mode for Check Point devices."""
-        return ''
+        return ""
 
-    def exit_config_mode(self, exit_config=''):
+    def exit_config_mode(self, exit_config=""):
         """No config mode for Check Point devices."""
-        return ''
+        return ""
+
+    def save_config(self, *args, **kwargs):
+        """Not Implemented"""
+        raise NotImplementedError
