@@ -1,4 +1,4 @@
 from __future__ import unicode_literals
 from netmiko.eltex.eltex_ssh import EltexSSH
 
-__all__ = ['EltexSSH']
+__all__ = ["EltexSSH"]
