@@ -1,4 +1,4 @@
 from __future__ import unicode_literals
 
 MAX_BUFFER = 65535
-BACKSPACE_CHAR = '\x08'
+BACKSPACE_CHAR = "\x08"
