@@ -13,8 +13,6 @@ from __future__ import unicode_literals
 import re
 import os
 import hashlib
-import time
-import io
 
 import scp
 
@@ -25,6 +23,7 @@ class SCPConn(object):
 
     Must close the SCP connection to get the file to write to the remote filesystem
     """
+
     def __init__(self, ssh_conn):
         self.ssh_ctl_chan = ssh_conn
         self.establish_scp_conn()
@@ -53,23 +52,34 @@ class SCPConn(object):
         self.scp_conn.close()
 
 
-class FileTransfer(object):
+class BaseFileTransfer(object):
     """Class to manage SCP file transfer and associated SSH control channel."""
-    def __init__(self, ssh_conn, source_file, dest_file, file_system=None, direction='put'):
+
+    def __init__(
+        self, ssh_conn, source_file, dest_file, file_system=None, direction="put"
+    ):
         self.ssh_ctl_chan = ssh_conn
         self.source_file = source_file
         self.dest_file = dest_file
         self.direction = direction
 
+        auto_flag = (
+            "cisco_ios" in ssh_conn.device_type
+            or "cisco_xe" in ssh_conn.device_type
+            or "cisco_xr" in ssh_conn.device_type
+        )
         if not file_system:
-            self.file_system = self.ssh_ctl_chan._autodetect_fs()
+            if auto_flag:
+                self.file_system = self.ssh_ctl_chan._autodetect_fs()
+            else:
+                raise ValueError("Destination file system not specified")
         else:
             self.file_system = file_system
 
-        if direction == 'put':
+        if direction == "put":
             self.source_md5 = self.file_md5(source_file)
             self.file_size = os.stat(source_file).st_size
-        elif direction == 'get':
+        elif direction == "get":
             self.source_md5 = self.remote_md5(remote_file=source_file)
             self.file_size = self.remote_file_size(remote_file=source_file)
         else:
@@ -83,8 +93,6 @@ class FileTransfer(object):
     def __exit__(self, exc_type, exc_value, traceback):
         """Context manager cleanup."""
         self.close_scp_chan()
-        if exc_type is not None:
-            raise exc_type(exc_value)
 
     def establish_scp_conn(self):
         """Establish SCP connection."""
@@ -95,23 +103,60 @@ class FileTransfer(object):
         self.scp_conn.close()
         self.scp_conn = None
 
-    def remote_space_available(self, search_pattern=r"bytes total \((.*) bytes free\)"):
+    def remote_space_available(self, search_pattern=r"(\d+) \w+ free"):
         """Return space available on remote device."""
-        remote_cmd = "dir {0}".format(self.file_system)
+        remote_cmd = "dir {}".format(self.file_system)
         remote_output = self.ssh_ctl_chan.send_command_expect(remote_cmd)
         match = re.search(search_pattern, remote_output)
+        if "kbytes" in match.group(0) or "Kbytes" in match.group(0):
+            return int(match.group(1)) * 1000
         return int(match.group(1))
 
+    def _remote_space_available_unix(self, search_pattern=""):
+        """Return space available on *nix system (BSD/Linux)."""
+        self.ssh_ctl_chan._enter_shell()
+        remote_cmd = "/bin/df -k {}".format(self.file_system)
+        remote_output = self.ssh_ctl_chan.send_command(
+            remote_cmd, expect_string=r"[\$#]"
+        )
+
+        # Try to ensure parsing is correct:
+        # Filesystem   1K-blocks  Used   Avail Capacity  Mounted on
+        # /dev/bo0s3f    1264808 16376 1147248     1%    /cf/var
+        remote_output = remote_output.strip()
+        output_lines = remote_output.splitlines()
+
+        # First line is the header; second is the actual file system info
+        header_line = output_lines[0]
+        filesystem_line = output_lines[1]
+
+        if "Filesystem" not in header_line or "Avail" not in header_line.split()[3]:
+            # Filesystem  1K-blocks  Used   Avail Capacity  Mounted on
+            msg = "Parsing error, unexpected output from {}:\n{}".format(
+                remote_cmd, remote_output
+            )
+            raise ValueError(msg)
+
+        space_available = filesystem_line.split()[3]
+        if not re.search(r"^\d+$", space_available):
+            msg = "Parsing error, unexpected output from {}:\n{}".format(
+                remote_cmd, remote_output
+            )
+            raise ValueError(msg)
+
+        self.ssh_ctl_chan._return_cli()
+        return int(space_available) * 1024
+
     def local_space_available(self):
         """Return space available on local filesystem."""
         destination_stats = os.statvfs(".")
         return destination_stats.f_bsize * destination_stats.f_bavail
 
-    def verify_space_available(self, search_pattern=r"bytes total \((.*) bytes free\)"):
+    def verify_space_available(self, search_pattern=r"(\d+) \w+ free"):
         """Verify sufficient space is available on destination file system (return boolean)."""
-        if self.direction == 'put':
+        if self.direction == "put":
             space_avail = self.remote_space_available(search_pattern=search_pattern)
-        elif self.direction == 'get':
+        elif self.direction == "get":
             space_avail = self.local_space_available()
         if space_avail > self.file_size:
             return True
@@ -119,43 +164,94 @@ class FileTransfer(object):
 
     def check_file_exists(self, remote_cmd=""):
         """Check if the dest_file already exists on the file system (return boolean)."""
-        if self.direction == 'put':
+        if self.direction == "put":
             if not remote_cmd:
-                remote_cmd = "dir {0}/{1}".format(self.file_system, self.dest_file)
+                remote_cmd = "dir {}/{}".format(self.file_system, self.dest_file)
             remote_out = self.ssh_ctl_chan.send_command_expect(remote_cmd)
             search_string = r"Directory of .*{0}".format(self.dest_file)
-            if 'Error opening' in remote_out:
+            if (
+                "Error opening" in remote_out
+                or "No such file or directory" in remote_out
+                or "Path does not exist" in remote_out
+            ):
                 return False
-            elif re.search(search_string, remote_out):
+            elif re.search(search_string, remote_out, flags=re.DOTALL):
                 return True
             else:
                 raise ValueError("Unexpected output from check_file_exists")
-        elif self.direction == 'get':
+        elif self.direction == "get":
+            return os.path.exists(self.dest_file)
+
+    def _check_file_exists_unix(self, remote_cmd=""):
+        """Check if the dest_file already exists on the file system (return boolean)."""
+        if self.direction == "put":
+            self.ssh_ctl_chan._enter_shell()
+            remote_cmd = "ls {}".format(self.file_system)
+            remote_out = self.ssh_ctl_chan.send_command(
+                remote_cmd, expect_string=r"[\$#]"
+            )
+            self.ssh_ctl_chan._return_cli()
+            return self.dest_file in remote_out
+        elif self.direction == "get":
             return os.path.exists(self.dest_file)
 
     def remote_file_size(self, remote_cmd="", remote_file=None):
         """Get the file size of the remote file."""
         if remote_file is None:
-            remote_file = self.dest_file
+            if self.direction == "put":
+                remote_file = self.dest_file
+            elif self.direction == "get":
+                remote_file = self.source_file
         if not remote_cmd:
-            remote_cmd = "dir {0}/{1}".format(self.file_system, remote_file)
-        remote_out = self.ssh_ctl_chan.send_command_expect(remote_cmd)
+            remote_cmd = "dir {}/{}".format(self.file_system, remote_file)
+        remote_out = self.ssh_ctl_chan.send_command(remote_cmd)
         # Strip out "Directory of flash:/filename line
         remote_out = re.split(r"Directory of .*", remote_out)
         remote_out = "".join(remote_out)
         # Match line containing file name
         escape_file_name = re.escape(remote_file)
-        pattern = r".*({0}).*".format(escape_file_name)
+        pattern = r".*({}).*".format(escape_file_name)
         match = re.search(pattern, remote_out)
         if match:
             line = match.group(0)
             # Format will be 26  -rw-   6738  Jul 30 2016 19:49:50 -07:00  filename
             file_size = line.split()[2]
-        if 'Error opening' in remote_out:
+        if "Error opening" in remote_out or "No such file or directory" in remote_out:
             raise IOError("Unable to find file on remote system")
         else:
             return int(file_size)
 
+    def _remote_file_size_unix(self, remote_cmd="", remote_file=None):
+        """Get the file size of the remote file."""
+        if remote_file is None:
+            if self.direction == "put":
+                remote_file = self.dest_file
+            elif self.direction == "get":
+                remote_file = self.source_file
+        remote_file = "{}/{}".format(self.file_system, remote_file)
+        if not remote_cmd:
+            remote_cmd = "ls -l {}".format(remote_file)
+
+        self.ssh_ctl_chan._enter_shell()
+        remote_out = self.ssh_ctl_chan.send_command(remote_cmd, expect_string=r"[\$#]")
+        self.ssh_ctl_chan._return_cli()
+
+        if "No such file or directory" in remote_out:
+            raise IOError("Unable to find file on remote system")
+
+        escape_file_name = re.escape(remote_file)
+        pattern = r"^.* ({}).*$".format(escape_file_name)
+        match = re.search(pattern, remote_out, flags=re.M)
+        if match:
+            # Format: -rw-r--r--  1 pyclass  wheel  12 Nov  5 19:07 /var/tmp/test3.txt
+            line = match.group(0)
+            file_size = line.split()[4]
+            return int(file_size)
+
+        raise ValueError(
+            "Search pattern not found for remote file size during SCP transfer."
+        )
+
     def file_md5(self, file_name):
         """Compute MD5 hash of file."""
         with open(file_name, "rb") as f:
@@ -164,7 +260,7 @@ class FileTransfer(object):
         return file_hash
 
     @staticmethod
-    def process_md5(md5_output, pattern=r"= (.*)"):
+    def process_md5(md5_output, pattern=r"=\s+(\S+)"):
         """
         Process the string to retrieve the MD5 hash
 
@@ -176,47 +272,48 @@ class FileTransfer(object):
         if match:
             return match.group(1)
         else:
-            raise ValueError("Invalid output from MD5 command: {0}".format(md5_output))
+            raise ValueError("Invalid output from MD5 command: {}".format(md5_output))
 
-    def compare_md5(self, base_cmd='verify /md5'):
-        """Compare md5 of file on network device to md5 of local file"""
-        if self.direction == 'put':
-            remote_md5 = self.remote_md5(base_cmd=base_cmd)
+    def compare_md5(self):
+        """Compare md5 of file on network device to md5 of local file."""
+        if self.direction == "put":
+            remote_md5 = self.remote_md5()
             return self.source_md5 == remote_md5
-        elif self.direction == 'get':
+        elif self.direction == "get":
             local_md5 = self.file_md5(self.dest_file)
             return self.source_md5 == local_md5
 
-    def remote_md5(self, base_cmd='verify /md5', remote_file=None):
-        """
-        Calculate remote MD5 and return the checksum.
+    def remote_md5(self, base_cmd="verify /md5", remote_file=None):
+        """Calculate remote MD5 and returns the hash.
 
         This command can be CPU intensive on the remote device.
         """
         if remote_file is None:
-            remote_file = self.dest_file
-        remote_md5_cmd = "{0} {1}{2}".format(base_cmd, self.file_system, remote_file)
-        dest_md5 = self.ssh_ctl_chan.send_command_expect(remote_md5_cmd, delay_factor=3.0)
+            if self.direction == "put":
+                remote_file = self.dest_file
+            elif self.direction == "get":
+                remote_file = self.source_file
+        remote_md5_cmd = "{} {}/{}".format(base_cmd, self.file_system, remote_file)
+        dest_md5 = self.ssh_ctl_chan.send_command(remote_md5_cmd, max_loops=1500)
         dest_md5 = self.process_md5(dest_md5)
         return dest_md5
 
     def transfer_file(self):
         """SCP transfer file."""
-        if self.direction == 'put':
+        if self.direction == "put":
             self.put_file()
-        elif self.direction == 'get':
+        elif self.direction == "get":
             self.get_file()
 
     def get_file(self):
         """SCP copy the file from the remote device to local system."""
-        self.scp_conn.scp_get_file(self.source_file, self.dest_file)
+        source_file = "{}/{}".format(self.file_system, self.source_file)
+        self.scp_conn.scp_get_file(source_file, self.dest_file)
         self.scp_conn.close()
 
     def put_file(self):
         """SCP copy the file from the local system to the remote device."""
-        destination = "{0}{1}".format(self.file_system, self.dest_file)
-        if ':' not in destination:
-            raise ValueError("Invalid destination file system specified")
+        destination = "{}/{}".format(self.file_system, self.dest_file)
         self.scp_conn.scp_transfer_file(self.source_file, destination)
         # Must close the SCP connection to get the file written (flush)
         self.scp_conn.close()
@@ -232,8 +329,8 @@ class FileTransfer(object):
         Defaults to Cisco IOS command
         """
         if cmd is None:
-            cmd = ['ip scp server enable']
-        elif not hasattr(cmd, '__iter__'):
+            cmd = ["ip scp server enable"]
+        elif not hasattr(cmd, "__iter__"):
             cmd = [cmd]
         self.ssh_ctl_chan.send_config_set(cmd)
 
@@ -244,164 +341,7 @@ class FileTransfer(object):
         Defaults to Cisco IOS command
         """
         if cmd is None:
-            cmd = ['no ip scp server enable']
-        elif not hasattr(cmd, '__iter__'):
+            cmd = ["no ip scp server enable"]
+        elif not hasattr(cmd, "__iter__"):
             cmd = [cmd]
         self.ssh_ctl_chan.send_config_set(cmd)
-
-
-class InLineTransfer(FileTransfer):
-    """Use TCL on Cisco IOS to directly transfer file."""
-    def __init__(self, ssh_conn, source_file=None, dest_file=None, file_system=None,
-                 direction='put', source_config=None):
-        if source_file and source_config:
-            msg = "Invalid call to InLineTransfer both source_file and source_config specified."
-            raise ValueError(msg)
-        if direction != 'put':
-            raise ValueError("Only put operation supported by InLineTransfer.")
-
-        self.ssh_ctl_chan = ssh_conn
-        if source_file:
-            self.source_file = source_file
-            self.source_config = None
-            self.source_md5 = self.file_md5(source_file)
-            self.file_size = os.stat(source_file).st_size
-        elif source_config:
-            self.source_file = None
-            self.source_config = source_config
-            self.source_md5 = self.config_md5(source_config)
-            self.file_size = len(source_config.encode('UTF-8'))
-        self.dest_file = dest_file
-        self.direction = direction
-
-        if not file_system:
-            self.file_system = self.ssh_ctl_chan._autodetect_fs()
-        else:
-            self.file_system = file_system
-
-    @staticmethod
-    def _read_file(file_name):
-        with io.open(file_name, "rt", encoding='utf-8') as f:
-            return f.read()
-
-    @staticmethod
-    def _tcl_newline_rationalize(tcl_string):
-        """
-        When using put inside a TCL {} section the newline is considered a new TCL
-        statement and causes a missing curly-brace message. Convert "\n" to "\r". TCL
-        will convert the "\r" to a "\n" i.e. you will see a "\n" inside the file on the
-        Cisco IOS device.
-        """
-        NEWLINE = r"\n"
-        CARRIAGE_RETURN = r"\r"
-        tmp_string = re.sub(NEWLINE, CARRIAGE_RETURN, tcl_string)
-        if re.search(r"[{}]", tmp_string):
-            msg = "Curly brace detected in string; TCL requires this be escaped."
-            raise ValueError(msg)
-        return tmp_string
-
-    def __enter__(self):
-        self._enter_tcl_mode()
-        return self
-
-    def __exit__(self, exc_type, exc_value, traceback):
-        _ = self._exit_tcl_mode()  # noqa
-        if exc_type is not None:
-            raise exc_type(exc_value)
-
-    def _enter_tcl_mode(self):
-        TCL_ENTER = 'tclsh'
-        cmd_failed = ['Translating "tclsh"', '% Unknown command', '% Bad IP address']
-        output = self.ssh_ctl_chan.send_command(TCL_ENTER, expect_string=r"\(tcl\)#",
-                                                strip_prompt=False, strip_command=False)
-        for pattern in cmd_failed:
-            if pattern in output:
-                raise ValueError("Failed to enter tclsh mode on router: {}".format(output))
-        return output
-
-    def _exit_tcl_mode(self):
-        TCL_EXIT = 'tclquit'
-        self.ssh_ctl_chan.write_channel("\r")
-        time.sleep(1)
-        output = self.ssh_ctl_chan.read_channel()
-        if '(tcl)' in output:
-            self.ssh_ctl_chan.write_channel(TCL_EXIT + "\r")
-        time.sleep(1)
-        output += self.ssh_ctl_chan.read_channel()
-        return output
-
-    def establish_scp_conn(self):
-        raise NotImplementedError
-
-    def close_scp_chan(self):
-        raise NotImplementedError
-
-    def local_space_available(self):
-        raise NotImplementedError
-
-    def file_md5(self, file_name):
-        """Compute MD5 hash of file."""
-        file_contents = self._read_file(file_name)
-        file_contents = file_contents + '\n'    # Cisco IOS automatically adds this
-        file_contents = file_contents.encode('UTF-8')
-        return hashlib.md5(file_contents).hexdigest()
-
-    def config_md5(self, source_config):
-        """Compute MD5 hash of file."""
-        file_contents = source_config + '\n'    # Cisco IOS automatically adds this
-        file_contents = file_contents.encode('UTF-8')
-        return hashlib.md5(file_contents).hexdigest()
-
-    def put_file(self):
-        curlybrace = r'{'
-        TCL_FILECMD_ENTER = 'puts [open "{}{}" w+] {}'.format(self.file_system,
-                                                              self.dest_file, curlybrace)
-        TCL_FILECMD_EXIT = '}'
-
-        if self.source_file:
-            file_contents = self._read_file(self.source_file)
-        elif self.source_config:
-            file_contents = self.source_config
-        file_contents = self._tcl_newline_rationalize(file_contents)
-
-        # Try to remove any existing data
-        self.ssh_ctl_chan.clear_buffer()
-
-        self.ssh_ctl_chan.write_channel(TCL_FILECMD_ENTER)
-        time.sleep(.25)
-        self.ssh_ctl_chan.write_channel(file_contents)
-        self.ssh_ctl_chan.write_channel(TCL_FILECMD_EXIT + "\r")
-
-        # This operation can be slow (depends on the size of the file)
-        max_loops = 400
-        sleep_time = 4
-        if self.file_size >= 2500:
-            max_loops = 1500
-            sleep_time = 12
-        elif self.file_size >= 7500:
-            max_loops = 3000
-            sleep_time = 25
-
-        # Initial delay
-        time.sleep(sleep_time)
-
-        # File paste and TCL_FILECMD_exit should be indicated by "router(tcl)#"
-        output = self.ssh_ctl_chan._read_channel_expect(pattern=r"\(tcl\)", max_loops=max_loops)
-
-        # The file doesn't write until tclquit
-        TCL_EXIT = 'tclquit'
-        self.ssh_ctl_chan.write_channel(TCL_EXIT + "\r")
-
-        time.sleep(1)
-        # Read all data remaining from the TCLSH session
-        output += self.ssh_ctl_chan._read_channel_expect(max_loops=max_loops)
-        return output
-
-    def get_file(self):
-        raise NotImplementedError
-
-    def enable_scp(self, cmd=None):
-        raise NotImplementedError
-
-    def disable_scp(self, cmd=None):
-        raise NotImplementedError
